library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
USE std.textio.ALL;

LIBRARY work;
USE work.MyStuff.ALL;

Entity LTC1407 is
    Port ( clk50   : in  STD_LOGIC; 
           cs      : in    STD_LOGIC;   -- chip-select: ADC starts with setting cs high
           frc     : in    STD_LOGIC;   -- free running conversion (high) or single shot (low)
           sck     : out   STD_LOGIC;
           conv    : out   STD_LOGIC;
           sdo     : in    STD_LOGIC;
           nWE     : out   STD_LOGIC;   -- negated write enable (memory)
           oChA    : out   STD_LOGIC_VECTOR (15 downto 0);   -- channel A result data from ADC
           oChB    : out   STD_LOGIC_VECTOR (15 downto 0);   -- channel B result data from ADC
           oaddr   : out   STD_LOGIC_VECTOR (19 downto 0));  -- self incrementing memory address in free running mode
end LTC1407;

Architecture std of LTC1407 is
Signal cnt   : integer range 0 to 34 := 0;
Signal sr    : std_logic_vector(31 downto 0);
Signal doADC : STD_LOGIC := '1';
Signal store : STD_LOGIC := '0';
Signal acnt  : integer range 0 to (2**20)-1 := 0;
BEGIN
   sck <= clk50;
   
   PROCESS (cs, clk50) 
      VARIABLE scratch : LINE;
   BEGIN
      if (cs = '0') then
         conv  <= '0';
         nWE   <= '1';
         store <= '0';
         doADC <= '1';
         cnt   <= 0;
      elsif rising_edge(clk50) then
         cnt <= cnt + 1;
         sr <= sr(sr'left-1 downto 0) & sdo; 
      
         case cnt is
            when 0 =>  if (doADC = '1') then conv <= '1'; end if;
            when 1 =>  conv <= '0';
            when 3 =>  if (store='1') then nWE <= '0'; end if;
            when 9 =>  if (store='1') then 
                          nWE  <= '1'; 
                          acnt <= acnt + 1; -- Autoincrement
                          store <= '0';
                       end if;
            when 32 => doADC <= frc;
                       if (doADC = '1') then
                          oChA <= x"0" & sr (27 downto 16);
                          oChB <= x"0" & sr (11 downto 0);
                          oaddr <= std_logic_vector(to_unsigned(acnt,20));
                          store <= '1';

                          write(scratch, STRING'("            7654321-7654321-7654321-76543210"));
                          writeline(output, scratch);
                          write(scratch, STRING'("current sr: "));
                          write(scratch, sr);
                          writeline(output, scratch);
                       end if;
            when 33 => cnt <= 0;
            when others => NULL;
         end case;
      end if;
   end process;
end std;

