LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE std.textio.ALL;

LIBRARY work;
USE work.MyStuff.ALL;

PACKAGE BODY MyStuff IS
   FUNCTION hex2int(i : in CHARACTER) RETURN INTEGER IS 
   BEGIN
      IF (i >= '0' AND i <= '9') THEN
         RETURN CHARACTER'pos(i) - 48;
      ELSE
         CASE i IS
         WHEN 'a' | 'A' => RETURN 10;
         WHEN 'b' | 'B' => RETURN 11;
         WHEN 'c' | 'C' => RETURN 12;
         WHEN 'd' | 'D' => RETURN 13;
         WHEN 'e' | 'E' => RETURN 14;
         WHEN 'f' | 'F' => RETURN 15;
         WHEN OTHERS => RETURN 0;
         END CASE;
      END IF;
   END FUNCTION;

   PROCEDURE parse(L : inout LINE; VALUE : inout INTEGER) IS
      CONSTANT OCT_TYPE : BIT_VECTOR(0 to 1) := "00";
      CONSTANT INT_TYPE : BIT_VECTOR(0 to 1) := "01";
      CONSTANT HEX_TYPE : BIT_VECTOR(0 to 1) := "10";
      CONSTANT BIT_TYPE : BIT_VECTOR(0 to 1) := "11";
      VARIABLE tmp : LINE;
      VARIABLE i   : NATURAL := 0;
      VARIABLE pos : NATURAL := 0;
      VARIABLE ok  : BOOLEAN := true;
      VARIABLE c   : CHARACTER;
      VARIABLE typ : BIT_VECTOR(0 to 1) := "01";
      VARIABLE buf : CHAR_VECTOR(0 to 32);
   BEGIN
      WHILE (ok) LOOP
         read(L, c, ok);

         EXIT WHEN NOT ok;

--       write(tmp, STRING'("got: c="));
--       write(tmp, c);
--       write(tmp, STRING'(", ok: "));
--       write(tmp, ok);
--       write(tmp, STRING'(", typ:"));
--       write(tmp, typ(0));
--       write(tmp, typ(1));
--       write(tmp, STRING'(", pos="));
--       write(tmp, pos);
--       writeline(output, tmp);

         IF (c >= '0' AND c <= '1') THEN
            buf(pos) := c; pos := pos + 1;
         ELSIF (c >= '2' AND c <= '7') THEN
            IF (typ /= BIT_TYPE) THEN
               buf(pos) := c; pos := pos + 1;
            END IF;   
         ELSIF (c >= '8' AND c <= '9') THEN
            IF (typ = INT_TYPE OR typ = HEX_TYPE) THEN
               buf(pos) := c; pos := pos + 1;
            END IF;
         ELSIF (pos = 0 AND (c = 'x' OR c = 'X')) THEN
            typ := HEX_TYPE;
         ELSIF (pos = 0 AND (c = 'o' OR c = 'O')) THEN
            typ := OCT_TYPE;
         ELSIF (pos = 0 AND (c = 'b' OR c = 'B')) THEN
            typ := BIT_TYPE;
         ELSIF ((c >= 'a' AND c <= 'f') OR (c >= 'A' AND c <= 'F')) THEN
            buf(pos) := c;
            pos := pos + 1;
         ELSE
            ok := false;
         END IF;
      END LOOP;
      value := 0;
      pos := pos - 1;
      CASE typ IS
      WHEN INT_TYPE => 
         FOR i IN 0 TO pos LOOP
--          write(tmp, STRING'("pos=")); write(tmp, pos);
--          write(tmp, STRING'(", i=")); write(tmp, i);
--          write(tmp, STRING'(", char=")); write(tmp, buf(i));
--          write(tmp, STRING'(", value0=")); write(tmp, value);
--          writeline(output, tmp);

            value := value + ((10 ** i) * (CHARACTER'pos(buf(pos - i)) - 48));
         END LOOP;
      WHEN OCT_TYPE =>
         FOR i IN 0 TO pos LOOP
--          write(tmp, STRING'("pos=")); write(tmp, pos);
--          write(tmp, STRING'(", i=")); write(tmp, i);
--          write(tmp, STRING'(", char=")); write(tmp, buf(i));
--          write(tmp, STRING'(", value0=")); write(tmp, value);
--          writeline(output, tmp);

            value := value + ((8 ** i) * (CHARACTER'pos(buf(pos - i)) - 48));
         END LOOP;
      WHEN HEX_TYPE =>
         FOR i IN 0 TO pos LOOP
--          write(tmp, STRING'("pos=")); write(tmp, pos);
--          write(tmp, STRING'(", i=")); write(tmp, i);
--          write(tmp, STRING'(", char=")); write(tmp, buf(i));
--          write(tmp, STRING'(", value0=")); write(tmp, value);
--          writeline(output, tmp);

            value := value + ((16 ** i) * hex2int(buf(pos - i)));
         END LOOP;
      WHEN BIT_TYPE =>
         FOR i IN 0 TO pos LOOP
--          write(tmp, STRING'("pos=")); write(tmp, pos);
--          write(tmp, STRING'(", i=")); write(tmp, i);
--          write(tmp, STRING'(", char=")); write(tmp, buf(i));
--          write(tmp, STRING'(", value0=")); write(tmp, value);
--          writeline(output, tmp);

            value := value + ((2 ** i) * (CHARACTER'pos(buf(pos - i)) - 48));
         END LOOP;
      WHEN OTHERS => NULL;
      END CASE;
--    write(tmp, STRING'("final value: ")); write(tmp, value);
--    writeline(output, tmp);
   END PROCEDURE;

   PROCEDURE write(L : inout LINE; VALUE : in std_logic_vector;
          	       JUSTIFIED: in SIDE := right;
	                FIELD: in WIDTH := 0) IS
      VARIABLE c  : character;
      VARIABLE mx : natural := value'length - 1;
   BEGIN
      FOR i IN 0 TO mx LOOP
         CASE value(mx - i) IS
         WHEN 'U' => c := 'U';
         WHEN 'X' => c := 'X';
         WHEN '0' => c := '0';
         WHEN '1' => c := '1';
         WHEN 'Z' => c := 'Z';
         WHEN 'W' => c := 'W';
         WHEN 'L' => c := 'L';
         WHEN 'H' => c := 'H';
         WHEN others => c := '-';
         END CASE;
         write(L, c);
      END LOOP;
   END PROCEDURE;

END MyStuff;
