library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Module to communicate with LTC1407
-- can be used for single-shot conversion (both channels) or 
-- free running conversion.
-- At free running conversion, memory address lines will be incremented on each sample
Entity fake_adc is
    Port ( channel_A  : in    STD_LOGIC_VECTOR (15 downto 0);
           channel_B  : in    STD_LOGIC_VECTOR (15 downto 0);
           clock      : in    STD_LOGIC; 
           aConv      : in    STD_LOGIC;   -- trigger to start conversion
           aData      : out   STD_LOGIC;   -- serial data from ADC
           sample     : out   STD_LOGIC ); -- just 4 debugging purpose 
end fake_adc;

Architecture sim of fake_adc is
begin
   -- generate serial and internal clock
   process (clock)
      VARIABLE send_Data : STD_LOGIC := '0';
      VARIABLE cnt       : NATURAL   := 0;
      VARIABLE sr        : STD_LOGIC_VECTOR(33 downto 0);
   begin      
      if (falling_edge(clock)) then -- ensure delay for client
         if (aConv = '1') then
            send_Data := '1';
            sample <= '1';
            sr := "000" & channel_A(11 downto 0) & "0000" & channel_B(11 downto 0) & "000";
            aData <= '0';
         end if;
         if (send_data = '1') then
            if (cnt < 34) then
               aData <= sr(sr'left);
               sr  := sr((sr'left -1) downto 0) & '0';
               cnt := cnt + 1;
            else   
               cnt := 0;
               send_Data := '0';
               sample <= '0';
            end if;
         end if;
      end if;
   end process;
end sim;
